package infra

import (
	"dcl_dispatch_server/src/package/util"
	_ "embed"
	"encoding/json"
	"errors"
	"fmt"
	"gopkg.in/yaml.v2"
)

type ApplicationYamlStruct struct {
	ApplicationName string       `yaml:"application-name"`
	TempDir         string       `yaml:"temp-dir"`
	Web             WebStruct    `yaml:"web"`
	Log             LogStruct    `yaml:"log"`
	Redis           RedisStruct  `yaml:"redis"`
	Kafka           KafkaStruct  `yaml:"kafka"`
	OssCicv         OssStruct    `yaml:"oss-cicv"`
	OssPji          OssStruct    `yaml:"oss-pji"`
	GpuNodeList     []GpuNode    `yaml:"gpu-node-list"`
	K8s             K8sStruct    `yaml:"k8s"`
	Docker          DockerStruct `yaml:"docker"`
}

type WebStruct struct {
	IpPrivate   string   `yaml:"ip-private"`
	Port        string   `yaml:"port"`
	RoutePrefix string   `yaml:"route-prefix"`
	Token       string   `yaml:"token"`
	WhiteList   []string `yaml:"white-list"`
}

type LogStruct struct {
	Dir    string `yaml:"dir"`
	Prefix string `yaml:"prefix"`
}

type RedisStruct struct {
	Addr     string `yaml:"addr"`
	Password string `yaml:"password"`
	Db       int    `yaml:"db"`
}

type KafkaStruct struct {
	Partition int32  `yaml:"partition"`
	Broker    string `yaml:"broker"`
}

type OssStruct struct {
	Type            string `yaml:"type"`
	IsUseCname      bool   `yaml:"is-use-cname"`
	Endpoint        string `yaml:"endpoint"`
	AccessKeyId     string `yaml:"access-key-id"`
	AccessKeySecret string `yaml:"access-key-secret"`
	BucketName      string `yaml:"bucket-name"`
}

type GpuNode struct {
	Hostname    string `yaml:"hostname"`
	Ip          string `yaml:"ip"`
	Parallelism int64  `yaml:"parallelism"`
}

type K8sStruct struct {
	PodYamlDir               string `yaml:"pod-yaml-dir"`
	VtdPodTemplateYamlPjisuv string `yaml:"vtd-pod-template-yaml-pjisuv"`
	VtdPodTemplateYamlPjibot string `yaml:"vtd-pod-template-yaml-pjibot"`
	AlgorithmTarTempDir      string `yaml:"algorithm-tar-temp-dir"`
	RegistryUri              string `yaml:"registry-uri"`
	NamespaceName            string `yaml:"namespace-name"`
	VtdImage                 string `yaml:"vtd-image"`
	VtdCommandPjisuv         string `yaml:"vtd-command-pjisuv"`
	VtdCommandPjibot         string `yaml:"vtd-command-pjibot"`
	CallbackUriCicv          string `yaml:"callback-uri-cicv"`
	CallbackUriPji           string `yaml:"callback-uri-pji"`
	PlatformType             string `yaml:"platform-type"`
}
type DockerStruct struct {
	Host string `yaml:"host"`
}

var (
	//go:embed application.yaml
	applicationYamlBytes []byte
	ApplicationYaml      ApplicationYamlStruct
)

func InitApplication() {
	_ = yaml.Unmarshal(applicationYamlBytes, &ApplicationYaml)
	fmt.Println("加载配置文件内容为：", ApplicationYaml)
	// 创建镜像下载目录
	util.CreateDir(ApplicationYaml.K8s.AlgorithmTarTempDir)
}
func JsonToGpuNode(jsonData string) (GpuNode, error) {
	// 创建一个 Person 类型的变量
	var taskCache GpuNode

	// 使用 json.Unmarshal 解析 JSON 字符串到结构体
	err := json.Unmarshal([]byte(jsonData), &taskCache)
	if err != nil {
		return GpuNode{}, errors.New("对象json " + jsonData + " 转对象失败错误信息为: " + fmt.Sprintf("%v", err))
	}
	return taskCache, nil
}

func GpuNodeToJson(gpuNode GpuNode) (string, error) {
	jsonData, err := json.MarshalIndent(gpuNode, "", "    ")
	if err != nil {
		return "", err
	}
	return string(jsonData), nil
}
